package io.repro.flutter;

import android.app.Activity;

import java.lang.ref.WeakReference;
import java.util.Date;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import io.repro.android.tracking.*;

import io.repro.android.Repro;
import io.repro.android.user.UserProfileGender;

/** ReproFlutterPlugin */
public class ReproFlutterPlugin implements MethodCallHandler {
  private static WeakReference<Activity> sActivityRef = null;

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "io.repro.flutter");
    channel.setMethodCallHandler(new ReproFlutterPlugin());
    sActivityRef = new WeakReference<>(registrar.activity());
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {

    // core/other

    if ("getUserID".equals(call.method)) {
      final String ret = getUserID();
      result.success(ret);
    } else if ("setUserID".equals(call.method)) {
      setUserID(call.arguments);
      result.success(null);
    } else if ("getDeviceID".equals(call.method)) {
      final String ret = getDeviceID();
      result.success(ret);
    } else if ("setLogLevel".equals(call.method)) {
      setLogLevel(call.arguments);
      result.success(null);
    } else if ("showInAppMessage".equals(call.method)) {
      showInAppMessage();
      result.success(null);
    } else if ("disableInAppMessageOnActive".equals(call.method)) {
      disableInAppMessageOnActive();
      result.success(null);
    } else if ("optIn".equals(call.method)) {
      optIn(call.arguments);
      result.success(null);
    }

    // user profile

    else if ("setStringUserProfile".equals(call.method)) {
      setStringUserProfile(call.arguments);
      result.success(null);
    } else if ("setIntUserProfile".equals(call.method)) {
      setIntUserProfile(call.arguments);
      result.success(null);
    } else if ("setDoubleUserProfile".equals(call.method)) {
      setDoubleUserProfile(call.arguments);
      result.success(null);
    } else if ("setDateUserProfile".equals(call.method)) {
      setDateUserProfile(call.arguments);
      result.success(null);
    } else if ("setUserGender".equals(call.method)) {
      setUserGender(call.arguments);
      result.success(null);
    } else if ("setUserEmailAddress".equals(call.method)) {
      setUserEmailAddress(call.arguments);
      result.success(null);
    }

    // event tracking

    else if ("track".equals(call.method)) {
      track(call.arguments);
      result.success(null);
    }
    else if ("trackViewContent".equals(call.method)) {
      trackViewContent(call.arguments);
      result.success(null);
    }
    else if ("trackSearch".equals(call.method)) {
      trackSearch(call.arguments);
      result.success(null);
    }
    else if ("trackAddToWishlist".equals(call.method)) {
      trackAddToWishlist(call.arguments);
      result.success(null);
    }
    else if ("trackAddToCart".equals(call.method)) {
      trackAddToCart(call.arguments);
      result.success(null);
    }
    else if ("trackInitiateCheckout".equals(call.method)) {
      trackInitiateCheckout(call.arguments);
      result.success(null);
    }
    else if ("trackAddPaymentInfo".equals(call.method)) {
      trackAddPaymentInfo(call.arguments);
      result.success(null);
    }
    else if ("trackPurchase".equals(call.method)) {
      trackPurchase(call.arguments);
      result.success(null);
    }
    else if ("trackShare".equals(call.method)) {
      trackShare(call.arguments);
      result.success(null);
    }
    else if ("trackCompleteRegistration".equals(call.method)) {
      trackCompleteRegistration(call.arguments);
      result.success(null);
    }
    else if ("trackLead".equals(call.method)) {
      trackLead(call.arguments);
      result.success(null);
    }

    else {
      result.notImplemented();
    }
  }

  // core/other

  private static String getUserID() {
    return Repro.getUserID();
  }

  private static void setUserID(final Object arguments) {
    Repro.setUserID((String) arguments);
  }

  private static String getDeviceID() {
    return Repro.getDeviceID();
  }

  private static void setLogLevel(final Object arguments) {
    final String logLevel = (String) arguments;
    if ("Debug".equals(logLevel)) {
      Repro.setLogLevel(android.util.Log.DEBUG);
    } else if ("Info".equals(logLevel)) {
      Repro.setLogLevel(android.util.Log.INFO);
    } else if ("Warn".equals(logLevel)) {
      Repro.setLogLevel(android.util.Log.WARN);
    } else if ("Error".equals(logLevel)) {
      Repro.setLogLevel(android.util.Log.ERROR);
    }
  }

  private static void showInAppMessage() {
    final Activity activity = sActivityRef.get();
    if (activity != null) {
      Repro.showInAppMessage(activity);
    }
  }

  private static void disableInAppMessageOnActive() {
    Repro.disableInAppMessageOnActive();
  }

  private static void optIn(final Object arguments) {
    Repro.optIn((Boolean) arguments);
  }

  // user profile

  private static void setStringUserProfile(final Object arguments) {
    final List list = (List) arguments;
    final String key = (String) list.get(0);
    final String value = (String) list.get(1);
    Repro.setStringUserProfile(key, value);
  }

  private static void setIntUserProfile(final Object arguments) {
    final List list = (List) arguments;
    final String key = (String) list.get(0);
    final Integer value = (Integer) list.get(1);
    Repro.setIntUserProfile(key, value);
  }

  private static void setDoubleUserProfile(final Object arguments) {
    final List list = (List) arguments;
    final String key = (String) list.get(0);
    final Double value = (Double) list.get(1);
    Repro.setDoubleUserProfile(key, value);
  }

  private static void setDateUserProfile(final Object arguments) {
    final List list = (List) arguments;
    final String key = (String) list.get(0);
    final Long value = (Long) list.get(1);
    Repro.setDateUserProfile(key, new Date(value));
  }

  private static void setUserGender(final Object arguments) {
    final String gender = (String) arguments;
    if ("Male".equals(gender)) {
      Repro.setUserGender(UserProfileGender.MALE);
    } else if ("Female".equals(gender)) {
      Repro.setUserGender(UserProfileGender.FEMALE);
    } else if ("Other".equals(gender)) {
      Repro.setUserGender(UserProfileGender.OTHER);
    }
  }

  private static void setUserEmailAddress(final Object arguments) {
    final String address = (String) arguments;
    Repro.setUserEmailAddress(address);
  }

  // event tracking

  private static void track(final Object argument) {
    final List list = (List) argument;
    final String name = (String) list.get(0);
    final Map map = (Map) list.get(1);

    Repro.track(name, map);
  }

  private static void trackViewContent(final Object argument) {
    final List list = (List) argument;
    final String contentId = (String) list.get(0);
    final Map map = (Map) list.get(1);
    final ViewContentProperties props = new ViewContentProperties();
    Object tmp;

    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackViewContent(contentId, props);
  }

  private static void trackSearch(final Object argument) {
    final Map map = (Map) argument;
    final SearchProperties props = new SearchProperties();
    Object tmp;

    tmp = map.get("content_id");        if (tmp instanceof String)  { props.setContentId((String) tmp); }
    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("search_string");     if (tmp instanceof String)  { props.setSearchString((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackSearch(props);
  }

  private static void trackAddToWishlist(final Object argument) {
    final Map map = (Map) argument;
    final AddToWishlistProperties props = new AddToWishlistProperties();
    Object tmp;

    tmp = map.get("content_id");        if (tmp instanceof String)  { props.setContentId((String) tmp); }
    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackAddToWishlist(props);
  }

  private static void trackAddToCart(final Object argument) {
    final List list = (List) argument;
    final String contentId = (String) list.get(0);
    final Map map = (Map) list.get(1);
    final AddToCartProperties props = new AddToCartProperties();
    Object tmp;

    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackAddToCart(contentId, props);
  }

  private static void trackInitiateCheckout(final Object argument) {
    final Map map = (Map) argument;
    final InitiateCheckoutProperties props = new InitiateCheckoutProperties();
    Object tmp;

    tmp = map.get("content_id");        if (tmp instanceof String)  { props.setContentId((String) tmp); }
    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("num_items");         if (tmp instanceof Integer) { props.setNumItems((Integer) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackInitiateCheckout(props);
  }

  private static void trackAddPaymentInfo(final Object argument) {
    final Map map = (Map) argument;
    final AddPaymentInfoProperties props = new AddPaymentInfoProperties();
    Object tmp;

    tmp = map.get("content_id");        if (tmp instanceof String)  { props.setContentId((String) tmp); }
    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackAddPaymentInfo(props);
  }

  private static void trackPurchase(final Object argument) {
    final List list = (List) argument;
    final String contentId = (String) list.get(0);
    final Double value = (Double) list.get(1);
    final String currency = (String) list.get(2);
    final Map map = (Map) list.get(3);
    final PurchaseProperties props = new PurchaseProperties();
    Object tmp;

    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("num_items");         if (tmp instanceof Integer) { props.setNumItems((Integer) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackPurchase(contentId, value, currency, props);
  }

  private static void trackShare(final Object argument) {
    final Map map = (Map) argument;
    final ShareProperties props = new ShareProperties();
    Object tmp;

    tmp = map.get("content_id");        if (tmp instanceof String)  { props.setContentId((String) tmp); }
    tmp = map.get("service_name");      if (tmp instanceof String)  { props.setServiceName((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackShare(props);
  }

  private static void trackCompleteRegistration(final Object argument) {
    final Map map = (Map) argument;
    final CompleteRegistrationProperties props = new CompleteRegistrationProperties();
    Object tmp;

    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("status");            if (tmp instanceof String)  { props.setStatus((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackCompleteRegistration(props);
  }

  private static void trackLead(final Object argument) {
    final Map map = (Map) argument;
    final LeadProperties props = new LeadProperties();
    Object tmp;

    tmp = map.get("value");             if (tmp instanceof Double)  { props.setValue((Double) tmp); }
    tmp = map.get("currency");          if (tmp instanceof String)  { props.setCurrency((String) tmp); }
    tmp = map.get("content_name");      if (tmp instanceof String)  { props.setContentName((String) tmp); }
    tmp = map.get("content_category");  if (tmp instanceof String)  { props.setContentCategory((String) tmp); }
    tmp = map.get("extras");            if (tmp instanceof Map)     { props.setExtras((Map) tmp); }

    Repro.trackLead(props);
  }
}