#import "ReproFlutterPlugin.h"
#import "Repro/Repro.h"

@implementation ReproFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"io.repro.flutter"
                                     binaryMessenger:[registrar messenger]];
    ReproFlutterPlugin* instance = [[ReproFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

// core/other

+ (NSString *)getUserID {
    return [Repro getUserID];
}

+ (void)setUserID:(id)arguments {
    NSString *userID = (NSString *)arguments;
    [Repro setUserID:userID];
}

+ (NSString *)getDeviceID {
    return [Repro getDeviceID];
}

+ (void)setLogLevel:(id)arguments {
    NSString *logLevel = (NSString *)arguments;
    if ([@"Debug" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelDebug];
    } else if ([@"Info" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelInfo];
    } else if ([@"Warn" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelWarn];
    } else if ([@"Error" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelError];
    }
}

+ (void)disableInAppMessageOnActive {
    [Repro disableInAppMessageOnActive];
}

+ (void)showInAppMessage {
    [Repro showInAppMessage];
}

+ (void)optIn:(id)arguments {
    [Repro optIn:[((NSNumber *)arguments) boolValue]];
}

// user profile

+ (void)setStringUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSString *value = (NSString *)array[1];
    [Repro setStringUserProfile:value forKey:key];
}

+ (void)setIntUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    [Repro setIntUserProfile:value.integerValue forKey:key];
}

+ (void)setDoubleUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    [Repro setDoubleUserProfile:value.doubleValue forKey:key];
}

+ (void)setDateUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    NSDate* date = [NSDate dateWithTimeIntervalSince1970:(value.longValue) / 1000.0];
    [Repro setDateUserProfile:date forKey:key];
}

+ (void)setUserGender:(id)arguments {
    NSString *gender = (NSString *)arguments;
    if ([@"Male" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderMale];
    } else if ([@"Female" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderFemale];
    } else if ([@"Other" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderOther];
    }
}

+ (void)setUserEmailAddress:(id)arguments {
    NSString *address = (NSString *)arguments;
    [Repro setUserEmailAddress:address];
}

// event tracking

+ (void)track:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *name = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];

    [Repro track:name properties:dict];
}

+ (void)trackViewContent:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];
    RPRViewContentProperties *props = [[RPRViewContentProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackViewContent:contentId properties:props];
}

+ (void)trackSearch:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRSearchProperties *props = [[RPRSearchProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"search_string"];       if ([tmp isKindOfClass:NSString.class])     { [props setSearchString:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackSearch:props];
}

+ (void)trackAddToWishlist:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRAddToWishlistProperties *props = [[RPRAddToWishlistProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddToWishlist:props];
}

+ (void)trackAddToCart:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];
    RPRAddToCartProperties *props = [[RPRAddToCartProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddToCart:contentId properties:props];
}

+ (void)trackInitiateCheckout:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRInitiateCheckoutProperties *props = [[RPRInitiateCheckoutProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"num_items"];           if ([tmp isKindOfClass:NSNumber.class])     { [props setNumItems:((NSNumber *)tmp).integerValue]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackInitiateCheckout:props];
}

+ (void)trackAddPaymentInfo:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRAddPaymentInfoProperties *props = [[RPRAddPaymentInfoProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddPaymentInfo:props];
}

+ (void)trackPurchase:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    NSString *currency = (NSString *)array[2];
    NSDictionary *dict = (NSDictionary *)array[3];
    RPRPurchaseProperties *props = [[RPRPurchaseProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"num_items"];           if ([tmp isKindOfClass:NSNumber.class])     { [props setNumItems:((NSNumber *)tmp).integerValue]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackPurchase:contentId value:value.doubleValue currency:currency properties:props];
}

+ (void)trackShare:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRShareProperties *props = [[RPRShareProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"service_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setServiceName:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackShare:props];
}

+ (void)trackCompleteRegistration:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRCompleteRegistrationProperties *props = [[RPRCompleteRegistrationProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"status"];              if ([tmp isKindOfClass:NSString.class])     { [props setStatus:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackCompleteRegistration:props];
}

+ (void)trackLead:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRLeadProperties *props = [[RPRLeadProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackLead:props];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {

    // core/other

    if ([@"getUserID" isEqualToString:call.method]) {
        NSString *userID = [ReproFlutterPlugin getUserID];
        result(userID);
    } else if ([@"setUserID" isEqualToString:call.method]) {
        [ReproFlutterPlugin setUserID:call.arguments];
        result(nil);
    } else if ([@"getDeviceID" isEqualToString:call.method]) {
        NSString *deviceID = [ReproFlutterPlugin getDeviceID];
        result(deviceID);
    } else if ([@"setLogLevel" isEqualToString:call.method]) {
        [ReproFlutterPlugin setLogLevel:call.arguments];
        result(nil);
    } else if ([@"disableInAppMessageOnActive" isEqualToString:call.method]) {
        [ReproFlutterPlugin disableInAppMessageOnActive];
        result(nil);
    } else if ([@"showInAppMessage" isEqualToString:call.method]) {
        [ReproFlutterPlugin showInAppMessage];
        result(nil);
    } else if ([@"optIn" isEqualToString:call.method]) {
        [ReproFlutterPlugin optIn:call.arguments];
        result(nil);
    }

    // user profile

    else if ([@"setStringUserProfile" isEqualToString:call.method]) {
        [ReproFlutterPlugin setStringUserProfile:call.arguments];
        result(nil);
    } else if ([@"setIntUserProfile" isEqualToString:call.method]) {
        [ReproFlutterPlugin setIntUserProfile:call.arguments];
        result(nil);
    } else if ([@"setDoubleUserProfile" isEqualToString:call.method]) {
        [ReproFlutterPlugin setDoubleUserProfile:call.arguments];
        result(nil);
    } else if ([@"setDateUserProfile" isEqualToString:call.method]) {
        [ReproFlutterPlugin setDateUserProfile:call.arguments];
        result(nil);
    } else if ([@"setUserGender" isEqualToString:call.method]) {
        [ReproFlutterPlugin setUserGender:call.arguments];
        result(nil);
    } else if ([@"setUserEmailAddress" isEqualToString:call.method]) {
        [ReproFlutterPlugin setUserEmailAddress:call.arguments];
        result(nil);
    }

    // event tracking

    else if ([@"track" isEqualToString:call.method]) {
        [ReproFlutterPlugin track:call.arguments];
        result(nil);
    } else if ([@"trackViewContent" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackViewContent:call.arguments];
        result(nil);
    } else if ([@"trackSearch" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackSearch:call.arguments];
        result(nil);
    } else if ([@"trackAddToWishlist" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackAddToWishlist:call.arguments];
        result(nil);
    } else if ([@"trackAddToCart" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackAddToCart:call.arguments];
        result(nil);
    } else if ([@"trackInitiateCheckout" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackInitiateCheckout:call.arguments];
        result(nil);
    } else if ([@"trackAddPaymentInfo" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackAddPaymentInfo:call.arguments];
        result(nil);
    } else if ([@"trackPurchase" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackPurchase:call.arguments];
        result(nil);
    } else if ([@"trackShare" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackShare:call.arguments];
        result(nil);
    } else if ([@"trackCompleteRegistration" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackCompleteRegistration:call.arguments];
        result(nil);
    } else if ([@"trackLead" isEqualToString:call.method]) {
        [ReproFlutterPlugin trackLead:call.arguments];
        result(nil);
    }

    else {
        result(FlutterMethodNotImplemented);
    }
}

@end
