import 'dart:async';

import 'package:flutter/services.dart';

enum LogLevel { debug, info, warn, error }

enum UserGender { male, female, other }

class Repro {
  static const MethodChannel _channel = const MethodChannel('io.repro.flutter');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<String> get userID async {
    final String userID = await _channel.invokeMethod('getUserID');
    return userID;
  }

  static set userID(final String userID) {
    assert(userID != null && userID != null);
    _channel.invokeMethod('setUserID', userID);
  }

  static Future<String> get deviceID async {
    final String deviceID = await _channel.invokeMethod('getDeviceID');
    return deviceID;
  }

  static set logLevel(final LogLevel logLevel) {
    String logLevelStr;
    switch (logLevel) {
      case LogLevel.debug:
        logLevelStr = 'Debug';
        break;
      case LogLevel.info:
        logLevelStr = 'Info';
        break;
      case LogLevel.warn:
        logLevelStr = 'Warn';
        break;
      case LogLevel.error:
        logLevelStr = 'Error';
        break;
    }
    _channel.invokeMethod('setLogLevel', logLevelStr);
  }

  static showInAppMessage() {
    _channel.invokeMethod('showInAppMessage');
  }

  static disableInAppMessageOnActive() {
    _channel.invokeMethod('disableInAppMessageOnActive');
  }

  static optIn(final bool endUserOptedIn) {
    assert(endUserOptedIn != null && endUserOptedIn != null);
    _channel.invokeMethod('optIn', endUserOptedIn);
  }

  static setStringUserProfile(final String key, final String value) {
    assert(key != null && value != null);
    _channel.invokeMethod('setStringUserProfile', [key, value]);
  }

  static setIntUserProfile(final String key, final int value) {
    assert(key != null && value != null);
    _channel.invokeMethod('setIntUserProfile', [key, value]);
  }

  static setDoubleUserProfile(final String key, final double value) {
    assert(key != null && value != null);
    _channel.invokeMethod('setDoubleUserProfile', [key, value]);
  }

  static setDateUserProfile(final String key, final DateTime value) {
    assert(key != null && value != null);
    _channel.invokeMethod('setDateUserProfile', [key, value.millisecondsSinceEpoch]);
  }

  static setUserGender(final UserGender userGender) {
    assert(userGender != null);
    String userGenderStr;
    switch (userGender) {
      case UserGender.male:
        userGenderStr = 'Male';
        break;
      case UserGender.female:
        userGenderStr = 'Female';
        break;
      case UserGender.other:
        userGenderStr = 'Other';
        break;
    }
    _channel.invokeMethod('setUserGender', userGenderStr);
  }

  static setUserEmailAddress(final String address) {
    assert(address != null);
    _channel.invokeMethod('setUserEmailAddress', address);
  }

  static track(final String event, [final Map<String, dynamic> properties = const {}]) {
    assert(event != null);
    _channel.invokeMethod('track', [event, properties]);
  }

  static trackViewContent(final String contentId, [final Map<String, dynamic> properties = const {}]) {
    assert(contentId != null);
    _channel.invokeMethod('trackViewContent', [contentId, properties]);
  }

  static trackSearch([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackSearch', properties);
  }

  static trackAddToWishlist([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackAddToWishlist', properties);
  }

  static trackAddToCart(final String contentId, [final Map<String, dynamic> properties = const {}]) {
    assert(contentId != null);
    _channel.invokeMethod('trackAddToCart', [contentId, properties]);
  }

  static trackInitiateCheckout([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackInitiateCheckout', properties);
  }

  static trackAddPaymentInfo([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackAddPaymentInfo', properties);
  }

  static trackPurchase(final String contentId, final double value, final String currency, [final Map<String, dynamic> properties = const {}]) {
    assert(contentId != null && value != null && currency != null);
    _channel.invokeMethod('trackPurchase', [contentId, value, currency, properties]);
  }

  static trackShare([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackShare', properties);
  }

  static trackCompleteRegistration([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackCompleteRegistration', properties);
  }

  static trackLead([final Map<String, dynamic> properties = const {}]) {
    _channel.invokeMethod('trackLead', properties);
  }
}
